package config

import (
	"flag"

	"github.com/BurntSushi/toml"
)

type Config struct {
	Debug  bool         `toml:"debug"`
	PGSQL  PGSQLConfig  `toml:"pgsql"`
	Server ServerConfig `toml:"server"`
	Chain  ChainConfig  `toml:"chain"`
}

type ChainConfig struct {
	RPC                 string `toml:"rpc"`
	UserContract        string `toml:"user_contract"`
	ExecutorContract    string `toml:"executor_contract"`
	XPrivateKey         string `toml:"x_private_key"`
	GasSenderPrivateKey string `toml:"gas_sender_private_key"`
	ExecutorPrivateKey  string `toml:"executor_private_key"`
	ModelPrice          int    `toml:"model_price"`
	AppPointContract    string `toml:"app_point_contract"`
}

type PGSQLConfig struct {
	Host        string `toml:"host"`
	Port        int    `toml:"port"`
	User        string `toml:"user"`
	Password    string `toml:"password"`
	Database    string `toml:"database"`
	MaxConn     int    `toml:"max_conn"`
	MaxIdleConn int    `toml:"max_idle_conn"`
	EnableLog   bool   `toml:"enable_log"`
	CertFile    string `toml:"cert_file"`
}

type ServerConfig struct {
	Listen string `toml:"listen"`
}

var confPath = flag.String("c", "config.toml", "config file path")

func New() (config *Config, err error) {
	config = new(Config)
	_, err = toml.DecodeFile(*confPath, config)
	return
}
