package dao

import (
	"contract_backend/contract/aon_executor"
	"contract_backend/contract/aon_user"
	"contract_backend/contract/erc20"
	"crypto/ecdsa"
	"math/big"

	"github.com/ethereum/go-ethereum/accounts/abi"
	"github.com/ethereum/go-ethereum/accounts/abi/bind"
	"github.com/ethereum/go-ethereum/common"
	"github.com/ethereum/go-ethereum/crypto"
)

func (d *Dao) Login(byteKey []byte, userId string, inviter string) (txHash common.Hash, err error) {
	instance, err := aon_user.NewAonUser(common.HexToAddress(d.c.Chain.UserContract), d.ethClient)
	if err != nil {
		return common.Hash{}, err
	}

	privateKey := crypto.ToECDSAUnsafe(byteKey)
	auth, err := bind.NewKeyedTransactorWithChainID(privateKey, d.chainId)
	if err != nil {
		return common.Hash{}, err
	}
	auth.GasPrice = d.gasPrice
	auth.GasLimit = 700000
	tx, err := instance.Login(auth, userId, inviter)
	if err != nil {
		return common.Hash{}, err
	}
	return tx.Hash(), nil
}

func (d *Dao) CheckApprove(byteKey []byte) (txHash common.Hash, err error) {
	instance, err := erc20.NewErc20(common.HexToAddress(d.c.Chain.AppPointContract), d.ethClient)
	if err != nil {
		return
	}
	privateKey := crypto.ToECDSAUnsafe(byteKey)
	user := crypto.PubkeyToAddress(privateKey.PublicKey)
	allowance, err := instance.Allowance(&bind.CallOpts{}, user, common.HexToAddress(d.c.Chain.ExecutorContract))
	if err != nil {
		return
	}
	if allowance.Cmp(abi.MaxUint256) == 0 {
		return
	}

	// 执行approve
	auth, err := bind.NewKeyedTransactorWithChainID(privateKey, d.chainId)
	if err != nil {
		return
	}
	auth.GasPrice = d.gasPrice
	auth.GasLimit = 100000

	tx, err := instance.Approve(auth, common.HexToAddress(d.c.Chain.ExecutorContract), abi.MaxUint256)
	if err != nil {
		return
	}
	return tx.Hash(), nil

}

func (d *Dao) ExecuteTask(privateKey *ecdsa.PrivateKey, executeId, userId, appId string, modelIds []string, price *big.Int) (txHash common.Hash, err error) {
	instance, err := aon_executor.NewAonExecutor(common.HexToAddress(d.c.Chain.ExecutorContract), d.ethClient)
	if err != nil {
		return common.Hash{}, err
	}

	auth, err := bind.NewKeyedTransactorWithChainID(privateKey, d.chainId)
	if err != nil {
		return common.Hash{}, err
	}
	auth.GasPrice = d.gasPrice
	auth.GasLimit = 1500000

	appId = "NYFiNxBEonlwYIrTQ2V8s9ks0I0ZPRoR1UVASanthVMA"
	modelIds = []string{"fUIqu5A2igp9nNJaEUMTod4amDFgIXWmHaTZNNbKo58A"}
	tx, err := instance.ExecuteByUserId(auth, executeId, userId, appId, modelIds, price)
	if err != nil {
		return common.Hash{}, err
	}
	return tx.Hash(), nil
}
