package util

import (
	"encoding/binary"
	"fmt"
	"strings"
	"testing"
	"time"

	"github.com/ethereum/go-ethereum/common"
	"github.com/ethereum/go-ethereum/crypto"
)

func TestXpriv(t *testing.T) {
	xpri := "xprv9xwX8JDK1LBoHbfUiif8tmaj5dTGEnrYrDat8RUNowmbz2hCD8N1wydNao1TBhgTzvF1gojUwekF4J28BNo4mCVW81BY3XUwDCQhscSMaC3"
	uid := "382a2aaf-572f-4a05-92b3-c05cf542bb63"
	k, _ := GetPrivateKeyByUserId(xpri, uid)
	fmt.Println(crypto.PubkeyToAddress(k.PublicKey))

}

func TestGenHDKey(t *testing.T) {

	xpub := "xpub6C9QTBQW4NDiwahEdPibQbLUtnhfny3E464dgDXGDkWFdqnkddYicVqUvaeAe8QrmXoSqXrcKMxszf8DJ2xzaQhth3nAgoBwjfC1J8jYwFS"
	// xpub := "xpub6C2ojpneBn4KHz1zaHpsyVHQMuJQbeDPbdkXywAUR43hXpjyQNcRv1ZQdvxnGmGKvXLoGPoN1G7cwfmW5CGZjPagpLnggXmqN52HhJk9F4B"

	ret, err := GetAddressByUserId("001b6462-43e6-4e8e-88b9-9636c8dece0d", xpub)
	t.Log(ret.String(), err)
	return
	// k, _ := GetPubKeyByPub(xpub, "0/1")
	// addr := crypto.PubkeyToAddress(*k)
	// t.Log(addr.String())

	userA := common.HexToAddress("0xAaf459E071637dE222D0a9e3b439704f478ed767")

	path := convertAddrToPath(userA.String())

	st := time.Now()
	k, _ := GetPubKeyByPub(xpub, path)
	addr2 := crypto.PubkeyToAddress(*k)
	t.Log(path, addr2.String())
	t.Log(time.Since(st))
	a, _ := GetDepositAddress(userA, xpub)
	t.Log(a.String())
	t.Log(addr2.String(), time.Since(st))

}

func convertAddrToPath(address string) string {
	addrBytes := common.HexToAddress(address).Bytes()
	// split to 8

	addrByteArray := make([]string, 7)
	for i := 0; i < 7; i++ {
		uint32Temp := uint32(0)
		if i == 6 {
			// fmt.Println("tt", tt)
			// binary.BigEndian.PutUint32(tt, addrByteArray[i])
			// fmt.Println("after tt", addrByteArray[i])
			uint32Temp = binary.BigEndian.Uint32(common.LeftPadBytes(addrBytes[i*3:i*3+2], 4))
		} else {
			// addrByteArray[i] = addrBytes[i*3 : i*3+3]
			// binary.BigEndian.PutUint32(temp4Byte(addrBytes[i*3:i*3+3]), addrByteArray[i])
			uint32Temp = binary.BigEndian.Uint32(common.LeftPadBytes(addrBytes[i*3:i*3+3], 4))
		}
		addrByteArray[i] = fmt.Sprintf("%d", uint32Temp)
	}

	fmt.Println(strings.Join(addrByteArray, "/"))
	return strings.Join(addrByteArray, "/")
}
